@@ -2,7 +2,7 @@
 
 	Written 1993-1998 by
 	Donald Becker, becker@cesdis.gsfc.nasa.gov, (driver core) and
-	David Hinds, dhinds@pcmcia.sourceforge.org (from his PC card code).
+	David Hinds, dahinds@users.sourceforge.net (from his PC card code).
 
 	This software may be used and distributed according to the terms of
 	the GNU Public License, incorporated herein by reference.
@@ -518,11 +518,11 @@ static void tc574_config(dev_link_t *link)
 		
 		/* Roadrunner only: Turn on the MII transceiver */
 		outw(0x8040, ioaddr + Wn3_Options);
-		udelay(1000);
+		mdelay(1);
 		outw(0xc040, ioaddr + Wn3_Options);
 		wait_for_completion(dev, TxReset);
 		wait_for_completion(dev, RxReset);
-		udelay(1000);
+		mdelay(1);
 		outw(0x8040, ioaddr + Wn3_Options);
 		
 		EL3WINDOW(4);
@@ -790,11 +790,11 @@ static void tc574_reset(struct net_device *dev)
 	
 	/* Roadrunner only: Turn on the MII transceiver. */
 	outw(0x8040, ioaddr + Wn3_Options);
-	udelay(1000);
+	mdelay(1);
 	outw(0xc040, ioaddr + Wn3_Options);
 	wait_for_completion(dev, TxReset);
 	wait_for_completion(dev, RxReset);
-	udelay(1000);
+	mdelay(1);
 	outw(0x8040, ioaddr + Wn3_Options);
 
 	/* Switch to the stats window, and clear all stats by reading. */
