@@ -129,10 +129,6 @@ char kernel_version[] = UTS_RELEASE;
 
 #if (LINUX_VERSION_CODE < 0x20123)
 #define test_and_set_bit(val, addr) set_bit(val, addr)
-#define le16_to_cpu(val) (val)
-#define cpu_to_le16(val) (val)
-#define le32_to_cpu(val) (val)
-#define cpu_to_le32(val) (val)
 typedef long spinlock_t;
 #define SPIN_LOCK_UNLOCKED 0
 #define spin_lock(lock)
@@ -1443,7 +1439,7 @@ static void speedo_interrupt(int irq, void *dev_instance, struct pt_regs *regs)
 #if LINUX_VERSION_CODE > 0x20127
 					sp->stats.tx_bytes += sp->tx_skbuff[entry]->len;
 #endif
-					dev_free_skb(sp->tx_skbuff[entry]);
+					dev_kfree_skb_irq(sp->tx_skbuff[entry]);
 					sp->tx_skbuff[entry] = 0;
 				} else if ((status & 0x70000) == CmdNOp)
 					sp->mc_setup_busy = 0;
