@@ -2,7 +2,7 @@
 
     PCMCIA Bulk Memory Services
 
-    bulkmem.c 1.36 2000/05/09 20:53:56
+    bulkmem.c 1.38 2000/09/25 19:29:51
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.1 (the "License"); you may not use this file
@@ -15,7 +15,7 @@
     rights and limitations under the License.
 
     The initial developer of the original code is David A. Hinds
-    <dhinds@pcmcia.sourceforge.org>.  Portions created by David A. Hinds
+    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
     are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.
 
     Alternatively, the contents of this file may be used under the
@@ -164,8 +164,7 @@ static void retry_erase(erase_busy_t *busy, u_int cause)
 	case MTD_WAITRDY:
 	    if (req.Status == MTD_WAITRDY)
 		insert_queue(&s->erase_busy, busy);
-	    busy->timeout.expires = jiffies + req.Timeout*HZ/1000;
-	    add_timer(&busy->timeout);
+	    mod_timer(&busy->timeout, jiffies + req.Timeout*HZ/1000);
 	    break;
 	}
     } else {
